@@ -13,7 +13,7 @@ import datetime
 
 from basic_logtools.filelog import LogFile
 
-from networktools.time import timestamp
+from networktools.time import timestamp, now
 from networktools.library import my_random_string
 '''
 ###############################################################################
@@ -134,7 +134,7 @@ class Eryo:
                     host=host,
                     port=port)
                 idc = await self.set_reader_writer(reader, writer)
-                if self.hard_beat(idc):
+                if self.heart_beat(idc):
                     self.status = True
                     self.logger.info("Conexion a %s realizada" % self.station)
                 else:
@@ -166,19 +166,27 @@ class Eryo:
 
     # callback for create server:
 
-    def hard_beat(self, idc):
-        if idc in self.clients:
+    def heart_beat(self, idc):
+        tnow = now()
+        if idc in self.clients.keys():
             reader = self.clients[idc]['reader']
             writer = self.clients[idc]['writer']
-            if writer.can_write_eof():
+            closing = writer.is_closing()
+            extra_info = writer.get_extra_info('peername')
+            at_eof = reader.at_eof()
+            if not closing and extra_info and not at_eof:
                 return True
             else:
-                self.logger.error("no hard_beat")
+                print("Fail heart beat to %s, at %s" %(self.station, tnow))
+                msg_error = "Closing %s, extra_info %s, at_eof %s" %(closing, extra_info,at_eof)
+                print(msg_error)
+                self.logger.error(msg_error)
+                self.logger.error("no heart_beat, at %s" %tnow)
                 self.close(idc)
                 self.status = False
                 return False
         else:
-            self.logger.error("no hard_beat, idc %s not client" % idc)
+            self.logger.error("no heart_beat, idc %s not client, code station %s" % (idc, self.station))
             self.close(idc)
             self.status = False
             return False
@@ -186,9 +194,9 @@ class Eryo:
     def info(self, idc):
         if self.clients:
             writer = self.clients[idc]['writer']
-            return writer.get_extra_info('peername')
+            return idc, writer.get_extra_info('peername')
         else:
-            return None
+            return idc, None
 
     def set_idc(self):
         """
@@ -219,17 +227,18 @@ class Eryo:
     def list_clients(self):
         for i in range(len(self.conss)):
             print(str(self.addrs[i]) + ":" + str(self.conns[i]))
-
-    def close(self, idc):
+            
+    async def close(self, idc):
         self.logger.error("La conexión se cerró en cliente %s" % idc)
         self.status = False
         reader = self.clients[idc]['reader']
         writer = self.clients[idc]['writer']
         writer.close()
+        await writer.wait_closed()
 
-    def stop(self):
+    async def stop(self):
         for idc in self.clients:
-            self.close(idc)
+            await self.close(idc)
 
     async def get_records(self):
         """
