@@ -113,7 +113,7 @@ class Gsof:
                     host=host,
                     port=port)
                 idc = await self.set_reader_writer(reader, writer)
-                if self.hard_beat(idc):
+                if self.heart_beat(idc):
                     self.status = True
                     self.logger.info("Conexion a %s realizada" % self.station)
                 else:
@@ -148,31 +148,37 @@ class Gsof:
 
     # callback for create server:
 
-    def hard_beat(self, idc):
+    def heart_beat(self, idc):
         tnow = now()
         if idc in self.clients.keys():
             reader = self.clients[idc]['reader']
             writer = self.clients[idc]['writer']
-            if writer.can_write_eof():
+            closing = writer.is_closing()
+            extra_info = writer.get_extra_info('peername')
+            at_eof = reader.at_eof()
+            if not closing and extra_info and not at_eof:
                 return True
             else:
-                print("Fail hard beat to %s, at %s" %(self.station, tnow))
-                self.logger.error("no hard_beat, at %s" %tnow)
+                print("Fail heart beat to %s, at %s" %(self.station, tnow))
+                msg_error = "Closing %s, extra_info %s, at_eof %s" %(closing, extra_info,at_eof)
+                print(msg_error)
+                self.logger.error(msg_error)
+                self.logger.error("no heart_beat, at %s" %tnow)
                 self.close(idc)
                 self.status = False
                 return False
         else:
-            self.logger.error("no hard_beat, idc %s not client, code station %s" % (idc, self.station))
+            self.logger.error("no heart_beat, idc %s not client, code station %s" % (idc, self.station))
             self.close(idc)
             self.status = False
             return False
-
+ 
     def info(self, idc):
         if self.clients:
             writer = self.clients[idc]['writer']
-            return writer.get_extra_info('peername')
+            return idc, writer.get_extra_info('peername')
         else:
-            return None
+            return idc, None
 
     def set_idc(self):
         """
@@ -204,16 +210,17 @@ class Gsof:
         for i in range(len(self.conss)):
             print(str(self.addrs[i]) + ":" + str(self.conns[i]))
 
-    def close(self, idc):
+    async def close(self, idc):
         self.logger.error("La conexión se cerró en cliente %s" % idc)
         self.status = False
         reader = self.clients[idc]['reader']
         writer = self.clients[idc]['writer']
         writer.close()
+        await writer.wait_closed()
 
-    def stop(self):
+    async def stop(self):
         for idc in self.clients:
-            self.close(idc)
+            await self.close(idc)
 
     async def get_message_header(self, idc):
         # receive data from source
